import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'constants/constants.dart';
import 'observer/safeXPay_payment_callback_observable.dart';

class AppHeader extends StatefulWidget {
  AppHeader(
      {required this.amount,
      required this.currency,
      required this.orderNo,
      this.showAmountCard = true,
      this.paymentId = '',
      this.transactionId = '',
      this.paymentStatus = '',
      this.statusMessage = '',
      this.status = false});

  final String orderNo;
  final double amount;
  final String currency;
  final bool showAmountCard;

  final String paymentId;
  final String transactionId;
  final String paymentStatus;
  final String statusMessage;
  final bool status;

  @override
  _AppHeader createState() => _AppHeader();
}

class _AppHeader extends State<AppHeader> {
  String?
      nwImage; // = 'https://pbs.twimg.com/profile_images/890513269131051008/x41AXaED_400x400.jpg';
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      alignment: AlignmentDirectional.topStart,
      fit: StackFit.loose,
      clipBehavior: Clip.hardEdge,
      children: [
        Container(
          height: UIConstants.headerHeight,
          width: MediaQuery.of(context).size.width,
          color: Colors.white,
          child: Column(
            children: [
              Expanded(
                flex: 60,
                child: Container(
                  color: Color(0xFFe0eefe),
                ),
              ),
              Expanded(
                flex: 40,
                child: Container(
                  color: Colors.white,
                ),
              ),
            ],
          ),
        ),
        Container(
          height: UIConstants.headerHeight,
          width: MediaQuery.of(context).size.width,
          padding: const EdgeInsets.fromLTRB(8, 48, 8, 8),
          child: Row(
            children: [
              /*Expanded(
                  flex: 4,
                  child: ),*/
              Card(
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(3)),
                color: Colors.white,
                elevation: 3.0,
                child: Container(
                    color: Colors.blue,
                    margin: const EdgeInsets.all(8),
                    width: 100,
                    height: 100,
                    child: nwImage == null
                        ? Image.asset(
                            'packages/safexpay/img/ic_safexpay.png',
                            fit: BoxFit.contain,
                            // height: 20,
                            // width: 20,
                          )
                        : Image.network(
                            nwImage!,
                            fit: BoxFit.contain,
                          )
                    // color: Colors.white,
                    ),
              ),
              Expanded(
                  flex: 6,
                  child: Column(
                    children: [
                      Expanded(
                        flex: 50,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: [
                            // const SizedBox(width: 12,),
                            Image.asset(
                              'packages/safexpay/img/safexpay_icon.png',
                              fit: BoxFit.contain,
                              height: 45,
                              // width: 20,
                            ),
                            const SizedBox(
                              width: 4,
                            ),
                            if (widget.showAmountCard)
                              GestureDetector(
                                child: Container(
                                  margin: EdgeInsets.only(bottom: 50),
                                  height: 40,
                                  width: 40,
                                  alignment: Alignment.topRight,
                                  child: Icon(Icons.close,
                                      size: 30, color: Colors.white),
                                ),
                                onTap: () {
                                  if (!widget.showAmountCard) {
                                    SafeXPayPaymentCallbackObservable
                                        callbackObservable =
                                        SafeXPayPaymentCallbackObservable();
                                    if (widget.status) {
                                      callbackObservable
                                          .notifyOnPaymentComplete(
                                              widget.orderNo,
                                              widget.transactionId,
                                              widget.paymentId,
                                              widget.paymentStatus);
                                    } else {
                                      callbackObservable
                                          .notifyOnInitiatePaymentFailure(
                                              widget.statusMessage);
                                    }
                                  }
                                  Navigator.of(context).pop();
                                },
                              )
                          ],
                        ),
                      ),
                      Expanded(
                        flex: 50,
                        child: widget.showAmountCard
                            ? Card(
                                color: Colors.white,
                                elevation: 3.0,
                                child: Container(
                                  padding: const EdgeInsets.all(8),
                                  // width: MediaQuery.of(context).size.height,
                                  // color: Colors.white,
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      Expanded(
                                        child: Text(
                                          "Total Amount",
                                          style: const TextStyle(
                                              color: Color(0xFF687982),
                                              fontWeight: FontWeight.w500,
                                              fontSize: 11.0),
                                        ),
                                      ),
                                      Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.spaceBetween,
                                        crossAxisAlignment:
                                            CrossAxisAlignment.end,
                                        children: [
                                          Text(
                                            "${widget.amount} ${widget.currency}",
                                            textDirection: TextDirection.ltr,
                                            style: const TextStyle(
                                                letterSpacing: 2,
                                                color: Color(0xFF687982),
                                                fontWeight: FontWeight.w700,
                                                fontSize: 18.0),
                                          ),
                                          // const SizedBox(width: 8,),
                                          Text(
                                            "Order No. ${widget.orderNo}",
                                            textDirection: TextDirection.ltr,
                                            style: const TextStyle(
                                                color: Color(0xFF687982),
                                                fontWeight: FontWeight.w500,
                                                fontSize: 10.0),
                                          ),
                                        ],
                                      )
                                    ],
                                  ),
                                ),
                              )
                            : Container(),
                      ),
                    ],
                  )),
            ],
          ),
        )
      ],
    );
  }
}
