/***********************************************
 * Created by Ashish Gupta on 7/19/2021
 * Mail-ID : ashishg@safexpay.com
 * Organization by SafexPay
 ***********************************************/
import 'package:flutter/material.dart';

// import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:modal_progress_hud_nsn/modal_progress_hud_nsn.dart';
import 'package:safexpay/AppHeader.dart';
import 'package:safexpay/constants/merchant_constants.dart';
import 'package:safexpay/constants/strings.dart';
import 'package:safexpay/constants/utility.dart';
import 'package:safexpay/logic/safex_pay_bloc.dart';
import 'package:safexpay/observer/safexpay_observable.dart';
import 'package:safexpay/utils/crypto_utils.dart';

import 'make_payment_webview.dart';
import 'observer/safexpay_observer.dart';

class MHSafeXPayGateway extends StatefulWidget {
  MHSafeXPayGateway(
      {required this.orderNo,
      required this.amount,
      required this.currency,
      required this.transactionType,
      required this.channel,
      required this.successUrl,
      required this.failureUrl,
      required this.countryCode,
      required this.pgDetails,
      required this.customerDetails,
      required this.cardDetails,
      required this.billDetails,
      required this.shipDetails,
      required this.itemDetails,
      required this.upiDetails,
      required this.otherDetails});

  final String orderNo;
  final double amount;
  final String currency;
  final String transactionType;
  final String channel;
  final String successUrl;
  final String failureUrl;
  final String countryCode;

  final String pgDetails;
  final String customerDetails;
  final String cardDetails;
  final String billDetails;
  final String shipDetails;
  final String itemDetails;
  final String upiDetails;
  final String otherDetails;

  @override
  _SafeXPayGatewayState createState() => _SafeXPayGatewayState();
}

class _SafeXPayGatewayState extends State<MHSafeXPayGateway>
    implements SafeXPayObserver {
  late SafeXPayBloc safeXPayBloc;
  int currentIndex = 0;
  SafeXPayObservable _safeXPayObservable = SafeXPayObservable();

  @override
  void initState() {
    _safeXPayObservable.register(this);
    safeXPayBloc = SafeXPayBloc(
        orderNo: widget.orderNo,
        amount: widget.amount,
        currency: widget.currency,
        transactionType: widget.transactionType,
        channel: widget.channel,
        successUrl: widget.successUrl,
        failureUrl: widget.failureUrl,
        countryCode: widget.countryCode);

    getMerchantBrandingDetails();
    generateChildrenList();
    super.initState();
  }

  Future<void> getMerchantBrandingDetails() async {
    bool isInternetAvailable = await Utility.isNetworkAvailable();

    if (isInternetAvailable) {
      safeXPayBloc.getBrandingDetails();
      //safeXPayBloc.getCardType('607385');
    } else {
      Utility.ackAlert(
          context, StringConstants.INTERNET_CONNECTIVITY_ERROR_MESSAGE);
    }
  }

  void generateChildrenList() {
    proceedPayment(context);
  }

  @override
  void dispose() {
    _safeXPayObservable.unRegister(this);
    safeXPayBloc.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: safeXPayBloc.scaffoldKey,
      body: StreamBuilder<bool>(
          stream: safeXPayBloc.loaderController.stream,
          builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
            bool inAsyncCall = false;
            if (snapshot.hasData && snapshot.data != null) {
              inAsyncCall = snapshot.data!;
            }
            return ModalProgressHUD(
              inAsyncCall: inAsyncCall,
              child: Column(
                children: [
                  AppHeader(
                    amount: widget.amount,
                    currency: widget.currency,
                    orderNo: widget.orderNo,
                  ),
                  /*Expanded(
                    child: MaterialApp(
                      debugShowCheckedModeBanner: false,
                      home: LoginScreen(
                        safeXPayBloc: safeXPayBloc,
                      ),
                    ),
                  ),*/
                  Expanded(
                    child: IndexedStack(
                      index: currentIndex,
                      //children: _children,
                    ),
                  )
                ],
              ),
            );
          }),
    );
  }

  @override
  void onIndexChanged(int index) {
    setState(() {
      currentIndex = index;
    });
  }

  Future<void> proceedPayment(BuildContext context) async {
    String strTransactionType = '${MerchantConstants.agId}'
        '|${MerchantConstants.merchantId}'
        '|${widget.orderNo}'
        '|${widget.amount}'
        '|${widget.countryCode}'
        '|${widget.currency}'
        '|${widget.transactionType}'
        '|${widget.successUrl}'
        '|${widget.failureUrl}'
        '|${widget.channel}';
    String? transactionDetails =
        CryptoUtils.encrypt(strTransactionType, MerchantConstants.merchantKey);

    String? pgDetails =
        CryptoUtils.encrypt(widget.pgDetails, MerchantConstants.merchantKey);

    String? cardDetails =
        CryptoUtils.encrypt(widget.cardDetails, MerchantConstants.merchantKey);
    String? custDetails = CryptoUtils.encrypt(
        widget.customerDetails, MerchantConstants.merchantKey);

    String? billDetails =
        CryptoUtils.encrypt(widget.billDetails, MerchantConstants.merchantKey);
    String? shipDetails =
        CryptoUtils.encrypt(widget.shipDetails, MerchantConstants.merchantKey);
    String? itemDetails =
        CryptoUtils.encrypt(widget.itemDetails, MerchantConstants.merchantKey);
    String? otherDetails =
        CryptoUtils.encrypt(widget.otherDetails, MerchantConstants.merchantKey);

    String? htmlContent = await safeXPayBloc.makePayment(
        transactionDetails,
        pgDetails,
        cardDetails,
        custDetails,
        billDetails,
        shipDetails,
        itemDetails,
        otherDetails,
        MerchantConstants.merchantId,
        context);
    debugPrint('html content is $htmlContent');

    if (htmlContent != null) {
      MaterialPageRoute route = MaterialPageRoute(
          builder: (BuildContext context) => MakePaymentWebViewScreen(
                makePaymentHtmlContent: htmlContent,
                webViewResponseUrl: widget.successUrl,
                isMerchantHosted: true,
              ));
      Navigator.pushReplacement(context, route);
    }
  }
}
