import 'package:flutter/material.dart';

// import 'package:modal_progress_hud/modal_progress_hud.dart';
import 'package:modal_progress_hud_nsn/modal_progress_hud_nsn.dart';
import 'package:safexpay/AppHeader.dart';
import 'package:safexpay/constants/strings.dart';
import 'package:safexpay/constants/utility.dart';
import 'package:safexpay/logic/safex_pay_bloc.dart';
import 'package:safexpay/login_screen.dart';
import 'package:safexpay/observer/safexpay_observable.dart';
import 'package:safexpay/wallets_screen.dart';

import 'NetBanking.dart';
import 'PaymentMode.dart';
import 'cardDetails.dart';
import 'observer/safexpay_observer.dart';

class SafeXPayGateway extends StatefulWidget {
  SafeXPayGateway(
      {required this.orderNo,
      required this.amount,
      required this.currency,
      required this.transactionType,
      required this.channel,
      required this.successUrl,
      required this.failureUrl,
      required this.countryCode});

  final String orderNo;
  final double amount;
  final String currency;
  final String transactionType;
  final String channel;
  final String successUrl;
  final String failureUrl;
  final String countryCode;
  bool userDetailsSet = false;
  String? name;
  String? emailId;
  String? mobile;

  void setUserDetails(
      {required String name, required String emailId, required String mobile}) {
    userDetailsSet = true;
    this.name = name;
    this.emailId = emailId;
    this.mobile = mobile;
  }

  @override
  _SafeXPayGatewayState createState() => _SafeXPayGatewayState();
}

class _SafeXPayGatewayState extends State<SafeXPayGateway>
    implements SafeXPayObserver {
  late SafeXPayBloc safeXPayBloc;
  late List<Widget> _children;
  int currentIndex = 0;
  SafeXPayObservable _safeXPayObservable = SafeXPayObservable();

  @override
  void initState() {
    _safeXPayObservable.register(this);
    safeXPayBloc = SafeXPayBloc(
        orderNo: widget.orderNo,
        amount: widget.amount,
        currency: widget.currency,
        transactionType: widget.transactionType,
        channel: widget.channel,
        successUrl: widget.successUrl,
        failureUrl: widget.failureUrl,
        countryCode: widget.countryCode);

    if (widget.userDetailsSet) {
      safeXPayBloc.saveUserDetails(
          name: widget.name!, emailId: widget.emailId!, mobile: widget.mobile!);
      safeXPayBloc.getMerchantPayModes();
    }
    getMerchantBrandingDetails();
    generateChildrenList();
    super.initState();
  }

  Future<void> getMerchantBrandingDetails() async {
    bool isInternetAvailable = await Utility.isNetworkAvailable();

    if (isInternetAvailable) {
      safeXPayBloc.getBrandingDetails();
      //safeXPayBloc.getCardType('607385');
    } else {
      Utility.ackAlert(
          context, StringConstants.INTERNET_CONNECTIVITY_ERROR_MESSAGE);
    }
  }

  Future<void> generateChildrenList() async {
    _children = <Widget>[
      LoginScreen(safeXPayBloc: safeXPayBloc),
      PaymentMode(safeXPayBloc: safeXPayBloc),
      CardDetails(safeXPayBloc: safeXPayBloc),
      NetBanking(safeXPayBloc: safeXPayBloc),
      WalletsScreen(safeXPayBloc: safeXPayBloc),
      // LoginScreen(safeXPayBloc: safeXPayBloc),
    ];
    if (widget.userDetailsSet) {
      final SafeXPayObservable safeXPayObservable = SafeXPayObservable();
      safeXPayObservable.notifyOnIndexChanged(1);
    }
  }

  @override
  void dispose() {
    _safeXPayObservable.unRegister(this);
    safeXPayBloc.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: safeXPayBloc.scaffoldKey,
      body: StreamBuilder<bool>(
          stream: safeXPayBloc.loaderController.stream,
          builder: (BuildContext context, AsyncSnapshot<bool> snapshot) {
            bool inAsyncCall = false;
            if (snapshot.hasData && snapshot.data != null) {
              inAsyncCall = snapshot.data!;
            }
            return ModalProgressHUD(
              inAsyncCall: inAsyncCall,
              child: Column(
                children: [
                  AppHeader(
                    amount: widget.amount,
                    currency: widget.currency,
                    orderNo: widget.orderNo,
                  ),
                  /*Expanded(
                    child: MaterialApp(
                      debugShowCheckedModeBanner: false,
                      home: LoginScreen(
                        safeXPayBloc: safeXPayBloc,
                      ),
                    ),
                  ),*/
                  Expanded(
                    child: IndexedStack(
                      index: currentIndex,
                      children: _children,
                    ),
                  )
                ],
              ),
            );
          }),
    );
  }

  @override
  void onIndexChanged(int index) {
    setState(() {
      currentIndex = index;
    });
  }
}
